"""Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


class Oracle:
    """Traces checking engine."""

    def __init__(self):
        """Default constructor."""
        self._trace_rec_map = {}

    def register_record(self, record, trace):
        """Registers log record of the specified trace in this oracle."""
        if record is None:
            return

        # If the record was in other traces, then it should be removed from everywhere
        removed = False
        for trace, records in self._trace_rec_map.iteritems():
            if record in records:
                self._trace_rec_map.get(trace).remove(record)
                if not removed:
                    removed = True

        if not removed:
            if trace not in self._trace_rec_map:
                self._trace_rec_map[trace] = []
            self._trace_rec_map[trace].append(record)

    def check(self):
        """Checks whether traces are equivalent."""
        match = True

        for records in self._trace_rec_map.values():
            if records and match:
                match = False

        if not match:
            print 'Non-matched records have been found:'
            for trace, records in self._trace_rec_map.iteritems():
                print 'Trace=%s' % trace.name()
                print ''
                print '%s' % '\n'.join([record.to_string() for record in records])
        else:
            print 'All the traces are equivalent.'
