"""Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

from UserDict import UserDict


class Record(UserDict):
    """Base class for log record."""

    TIME = "time"
    SCALE = "scale"

    def __init__(self, time, scale):
        """Default constructor."""
        UserDict.__init__(self)
        self[self.TIME] = time
        self[self.SCALE] = scale

    def get_time(self):
        """Returns the timestamp (decimal value)."""
        return self[self.TIME]

    def get_scale(self):
        """Returns the unit that indicates the timestamp is not related to real time, but an increasing count."""
        return self[self.SCALE]

    def to_string(self):
        """Returns the string representation of this record, None fields are removed."""
        data = []
        for value in self.values():
            if value is not None:
                data.append(value)

        return ' '.join(data)
